#include <algorithm>
#include <vector>
#include <stdexcept>
using namespace std;

/**
  HashCode d'un (ix,iy) dans le tableau unidimensionnel
  @param[in] ix - numero colonne
  @param[iy] iy - numero ligne
*/
inline unsigned Echiquier::hashCode(unsigned ix, unsigned iy) const
{
  return iy * sizeN() + ix;
}

/**
  Constructeur par defaut
*/
Echiquier::Echiquier()
: m_vals(), m_dim(0)
{}

/**
  Constructeur issu d'une dimension
  @param[in] n - dimension
*/
Echiquier::Echiquier(unsigned n)
: m_vals(n*n), m_dim(n)
{}

/**
  Constructeur issu d'une dimension et d'une valeur
  @param[in] n - dimension
  @param[in] vinit - valeur d'initialisation
*/
Echiquier::Echiquier(unsigned n, int vinit)
: m_vals(n*n, vinit), m_dim(n)
{}

/**
  Accesseur de la valeur en (x,y)
  @param[in] x - numero colonne
  @param[iy] y - numero ligne
  @return la valeur en (x,y)
*/
inline int Echiquier::get(unsigned x, unsigned y) const
{
  return m_vals[ hashCode(x, y) ];
}

/**
  Accesseur de la valeur en une Position
  @param[in] pos - une Position
  @return la valeur en pos
*/
inline int Echiquier::get(const Position& pos) const
{
  return m_vals[ hashCode(pos.getX(), pos.getY()) ];
}

/**
  Fixe la valeur en une position
  @param[in] pos - une Position
  @param[in] val - la nouvelle valeur
*/
inline void Echiquier::set(const Position& pos, int val)
{
  m_vals[ hashCode(pos.getX(), pos.getY()) ] = val;
}

/**
  Accesseur de la dimension
  @return la dimension
*/
inline unsigned Echiquier::sizeN() const
{
  return m_dim;
}

/**
  Teste si une position est dans l'echiquier
  @param[in] pos - une Position
  @return Vrai si pos dans dans l'echiquier
*/
bool Echiquier::contient(const Position& pos) const
{
  return (0 <= pos.getX() && pos.getX() < static_cast<int>(sizeN())
       && 0 <= pos.getY() && pos.getY() < static_cast<int>(sizeN()));

}

/**
  Re-initialise l'echiquier à une valeur
  @param[in] val - valeur d'initialisation
*/
void Echiquier::reset(int val)
{
  fill(m_vals.begin(), m_vals.end(), val);
}

