#include <iostream>
using namespace std;
/**
  Constructeur par defaut: position d'origine (0,0)
*/
Position::Position()
: m_x(0), m_y(0)
{}

/**
  Constructeur normal
  @param[in] xx - position x
  @param[in] yy - position y
*/
Position::Position(int xx, int yy)
: m_x(xx), m_y(yy)
{}

/**
  Accesseur de la premiere composante
  @return premiere composante
*/
int Position::getX() const
{
  return m_x;
}

/**
  Accesseur de la deuxieme composante
  @return deuxieme composante
*/
int Position::getY() const
{
  return m_y;
}

/**
  Methode d'affectation
  @param[in] xx - position x
  @param[in] yy - position y
*/
void Position::assign(int xx, int yy)
{
  m_x = xx;
  m_y = yy;
}

/**
  Lit une Position depuis un flux d'entree
  Les donnees extraites sont analysees pour verifier
  qu'elles sont au bon format: x, (x) ou (x,y)
  @param[in,out] is - flux d'entree
  @param[out] pos - une Position
  @return le status de l'operation (vrai si OK - faux sinon)
*/
bool lirePosition(istream& is, Position& pos)
{
  // Reinitialise l'etat du flux et ignore les espaces
  is.clear();
  is >> std::ws;
  // Si probleme: on retourne
  if (not is.good())
  {
    return false;
  }
  // Examine le prochain caractere: si '(' alors analyse
  // une position; sinon pas de parenthese ouvrante:
  // ce sera un simple entier (y == 0)
  int xx, yy;
  if (is.peek() == '(')
  {
    // Mange la parenthese ouvrante
    char c; is >> c;
    // Ne devrait jamais arriver sauf si probleme systeme
    if (not is.good())
    {
      is.clear(is.rdstate() | std::ios::failbit);
      return false;
    }
    // Lit la premiere composante
    is>>xx;
    // Si l'analyse echoue, il y avait une donnee erronee
    if (not is.good())
    {
      is.clear(is.rdstate() | std::ios::badbit);
      return false;
    }
    // Retire les eventuels espaces
    is>>std::ws;
    // Si l'analyse echoue, il y avait une donnee erronee
    if (not is.good())
    {
      is.clear(is.rdstate() | std::ios::badbit);
      return false;
    }
    // Examine le caractere suivant
    char nextChar = is.peek();
    // Si c'est une virgule, il y a la deuxieme composante
    if (nextChar == ',')
    {
      // Mange le caractere:
      is>>c;
      // Ne devrait jamais arriver sauf si probleme systeme
      if (not is.good())
      {
        is.clear(is.rdstate() | std::ios::failbit);
        return false;
      }
      // Lit la deuxieme composante
      is>>yy;
      // Si l'analyse echoue, il y avait une donnee erronee
      if (not is.good())
      {
        is.clear(is.rdstate() | std::ios::badbit);
        return false;
      }
      // Retire les eventuels espaces
      is>>std::ws;
      // Si l'analyse echoue, il y avait une donnee erronee
      if (not is.good())
      {
        is.clear(is.rdstate() | std::ios::badbit);
        return false;
      }
      // Examine le caractere suivant
      nextChar = is.peek();
    }
    else
    {
      // Le prochain caractere n'est pas une virgule; il n'y
      // aura pas de deuxieme composante. On s'assure qu'elle
      // est correctement initialisee
      yy = 0;
    }
    // L'absence de parenthese fermante est une erreur mais
    // on a reussi a lire une valeur complete que l'on retourne;
    // on retire la parenthese si elle est presente
    if (nextChar != ')')
    {
      is.clear(is.rdstate() | std::ios::badbit);
    }
    else
    {
      is>>c;
      // Ne devrait jamais arriver sauf si probleme systeme
      if (not is.good())
      {
        is.clear(is.rdstate() | std::ios::failbit);
        return false;
      }
    }
  }
  else
  {
    // Lit la premiere composante
    is>>xx;
    // Si l'analyse echoue, erreur grave
    if (not is.good())
    {
      return false;
    }
    // Sinon initialise la deuxieme composante
    yy = 0;
  }
  // Ici les donnees ont ete correctement lues: modifie l'objet
  pos.assign(xx, yy);
  return true;
}

/**
  Saisit d'une Position depuis le clavier
  @param[out] pos - une Position
*/
void saisirPosition(Position& pos)
{
  bool b = lirePosition(cin, pos);
}

/**
  Ecrit une Position sur un flux de sortie, format (x,y)
  @param[in,out] os - flux de sortie
  @param[in] pos - une Position
  @return le status de l'operation (vrai si OK - faux sinon)
*/
bool ecrirePosition(ostream& os, const Position& pos)
{
  os<<"("<<pos.getX()<<","<<pos.getY()<<")";
  return os.good();
}

/**
  Affiche une Position sur l'ecran
  @param[in] pos - une Position
*/
void afficherPosition(const Position& pos)
{
  bool b = ecrirePosition(cout, pos);
}

/**
  Surdefinit l'operateur de flux d'entree pour les Position
  @param[in,out] is - flux d'entree
  @param[out] pos - une Position
  @return le flux modifie
*/
istream& operator>>(istream& is, Position& obj)
{
  lirePosition(is, obj);
  return is;
}

/**
  Surdefinit l'operateur de flux de sortie pour les Position
  @param[in,out] os - flux de sortie
  @param[in] pos - une Position
  @return le flux modifie
*/
ostream& operator<<(ostream& os, const Position& obj)
{
  ecrirePosition(os, obj);
  return os;
}

